@@ -3,11 +3,11 @@
 import os
 from tg import TGController
 from tg import expose, flash, require, url, lurl, request, redirect, validate, tmpl_context, config
-from tg.decorators import before_render
+from tg.decorators import before_render, cached_property
 from tg.i18n import ugettext as _, lazy_ugettext as l_
 from repoze.what import predicates
 
-from smallpress.model import DBSession, Article, Attachment, Tagging
+from smallpress.model import DBSession, Article, Attachment, Tagging, Blog
 from smallpress.helpers import *
 from smallpress.lib.forms import ArticleForm, UploadForm
 from tgext.datahelpers.validators import SQLAEntityConverter
@@ -20,12 +20,19 @@ from tw.forms.validators import UnicodeString
 from datetime import datetime
 from tgext.ajaxforms.ajaxform import formexpose, spinner_icon
 from tgext.tagging import TaggingController
-import whoosh
-from whoosh.query import *
+from tgext.crud import EasyCrudRestController
+
+try:
+    import whoosh
+    from whoosh.query import *
+    whoosh_enabled = True
+except ImportError:
+    whoosh_enabled = False
 
 article_form = ArticleForm()
 upload_form = UploadForm()
 articles_table = DataGrid(fields=[(l_('Actions'), lambda row:link_buttons(row, 'edit', 'delete', 'hide', 'publish')),
+                                  (l_('Blog'), lambda row:row.blog and row.blog.name),
                                   (l_('Title'), lambda row:HTML.a(row.title,
                                                                   href=plug_url('smallpress', '/view/%s'%row.uid,
                                                                                 lazy=True))),
@@ -39,15 +46,33 @@ attachments_table = DataGrid(fields=[(l_('Actions'), lambda row:link_buttons(row
 def inject_css(*args, **kw):
     CSSLink(link='/_pluggable/smallpress/css/style.css').inject()
 
+class BlogsController(EasyCrudRestController):
+    allow_only = predicates.in_group('smallpress')
+    title = "Manage Blogs"
+    model = Blog
+
+    __form_options__ = {
+        '__hide_fields__' : ['uid'],
+        '__omit_fields__' : ['articles']
+    }
+
+    __table_options__ = {
+        '__omit_fields__' : ['uid', 'articles']
+    }
+
 class RootController(TGController):
     tagging = TaggingController(model=Article, session=DBSession, allow_edit=None)
     tagging.search = before_render(inject_css)(tagging.search)
 
+    @cached_property
+    def blogs(self):
+        return BlogsController(DBSession.wrapped_session)
+
     @expose('genshi:smallpress.templates.index')
-    def index(self, *args, **kw):
-        articles = Article.get_published().all()
+    def index(self, blog='', *args, **kw):
+        articles = Article.get_published(blog).all()
         tags = Tagging.tag_cloud_for_set(Article, articles).all()
-        return dict(articles=articles, tags=tags)
+        return dict(articles=articles, tags=tags, blog=blog)
 
     @expose('genshi:smallpress.templates.article')
     @validate(dict(article=SQLAEntityConverter(Article)), error_handler=index)
@@ -68,18 +93,21 @@ class RootController(TGController):
 
     @require(predicates.in_group('smallpress'))
     @expose('genshi:smallpress.templates.manage')
-    def manage(self, *args, **kw):
-        articles = DBSession.query(Article).order_by(Article.publish_date.desc())
+    def manage(self, blog='', *args, **kw):
+        articles = Article.get_all(blog)
         return dict(table=articles_table, articles=articles,
-                    create_action=self.mount_point+'/new')
+                    create_action=self.mount_point+'/new/'+blog)
 
     @require(predicates.in_group('smallpress'))
     @expose('genshi:smallpress.templates.edit')
-    def new(self, **kw):
+    def new(self, blog=None, **kw):
         attachments_table.register_resources()
 
+        if blog:
+            blog = DBSession.query(Blog).filter_by(name=blog).first()
+
         if 'uid' not in kw:
-            article = Article(author=request.identity['user'])
+            article = Article(author=request.identity['user'], blog=blog and blog)
             DBSession.add(article)
             DBSession.flush()
 
@@ -94,10 +122,10 @@ class RootController(TGController):
             'publish_date':kw['publish_date'],
             'tags':kw.get('tags', ''),
             'description':kw.get('description', ''),
-            'content':kw.get('content', '')
+            'content':kw.get('content', ''),
         }
 
-        return dict(article=article, value=value,
+        return dict(article=article, value=value, blog=blog and blog.name or '',
                     form=article_form, action=url(self.mount_point+'/save'),
                     upload_form=upload_form, upload_action=url(self.mount_point+'/attach'))
 
@@ -116,7 +144,7 @@ class RootController(TGController):
             'content':article.content
         }
 
-        return dict(article=article, value=value,
+        return dict(article=article, value=value, blog=article.blog and article.blog.name or '',
                     form=article_form, action=url(self.mount_point+'/save'),
                     upload_form=upload_form, upload_action=url(self.mount_point+'/attach'))
 
@@ -129,10 +157,11 @@ class RootController(TGController):
         article.description = kw['description']
         article.content = kw['content']
         article.publish_date = datetime.strptime(kw['publish_date'], '%Y-%m-%d %H:%M')
+        article.update_date = datetime.now()
         Tagging.set_tags(article, kw['tags'])
 
         flash(_('Articles successfully saved'))
-        return redirect(self.mount_point+'/manage')
+        return redirect(self.mount_point+'/manage/'+article.blog_name)
 
     @require(predicates.in_group('smallpress'))
     @formexpose(upload_form, 'smallpress.templates.attachments')
@@ -173,7 +202,7 @@ class RootController(TGController):
     def publish(self, article):
         article.published=True
         flash(_('Article published'))
-        return redirect(self.mount_point+'/manage')
+        return redirect(self.mount_point+'/manage/'+article.blog_name)
 
     @require(predicates.in_group('smallpress'))
     @validate(dict(article=SQLAEntityConverter(Article)), error_handler=manage)
@@ -181,7 +210,7 @@ class RootController(TGController):
     def hide(self, article):
         article.published=False
         flash(_('Article hidden'))
-        return redirect(self.mount_point+'/manage')
+        return redirect(self.mount_point+'/manage/'+article.blog_name)
 
     @require(predicates.in_group('smallpress'))
     @validate(dict(article=SQLAEntityConverter(Article)), error_handler=manage)
@@ -189,22 +218,25 @@ class RootController(TGController):
     def delete(self, article):
         DBSession.delete(article)
         flash(_('Article successfully removed'))
-        return redirect(self.mount_point+'/manage')
+        return redirect(self.mount_point+'/manage/'+article.blog_name)
 
     @expose('genshi:smallpress.templates.index')
     @validate(dict(text=UnicodeString(not_empty=True)), error_handler=index)
     def search(self, text=None):
         articles = []
 
-        index_path = config.get('smallpress_whoosh_index', '/tmp/smallpress_whoosh')
-        ix = whoosh.index.open_dir(index_path)
-        with ix.searcher() as searcher:
-            query = Or([Term("content", text),
-                        Term("title", text),
-                        Term("description", text)])
-            found = searcher.search(query)
-            if len(found):
-                articles = Article.get_published().filter(Article.uid.in_([e['uid'] for e in found])).all()
+        if whoosh_enabled:
+            index_path = config.get('smallpress_whoosh_index', '/tmp/smallpress_whoosh')
+            ix = whoosh.index.open_dir(index_path)
+            with ix.searcher() as searcher:
+                query = Or([Term("content", text),
+                            Term("title", text),
+                            Term("description", text)])
+                found = searcher.search(query)
+                if len(found):
+                    articles = Article.get_published().filter(Article.uid.in_([e['uid'] for e in found])).all()
+        else:
+            articles = Article.get_published().filter(Article.content.like('%'+text+'%')).all()
 
         tags = Tagging.tag_cloud_for_set(Article).all()
-        return dict(articles=articles, tags=tags)
+        return dict(articles=articles, tags=tags, blog='')
