@@ -14,5 +14,5 @@ Tag, Tagging = tgext.tagging.setup_model({'DBSession':DBSession,
                                           'User':app_model.User,
                                           'metadata':DeclarativeBase.metadata})
 
-from models import Article, Attachment
+from models import Article, Attachment, Blog
 
